import Adapter from '@ember-data/adapter';
import DS, { ModelSchema } from 'ember-data';
import ModelRegistry from 'ember-data/types/registries/model';
import RSVP from 'rsvp';
import Store from '@ember-data/store';
import { CollectionReference, Firestore, Query, WriteBatch } from 'firebase/firestore';
import FirestoreDataManager from 'ember-cloud-firestore-adapter/services/-firestore-data-manager';
export interface AdapterOption {
    isRealtime?: boolean;
    queryId?: string;
    buildReference?(db: Firestore): CollectionReference;
    filter?(db: CollectionReference): Query;
    include?(batch: WriteBatch, db: Firestore): void;
    [key: string]: unknown;
}
interface Snapshot extends DS.Snapshot {
    adapterOptions: AdapterOption;
}
interface SnapshotRecordArray extends DS.SnapshotRecordArray<keyof ModelRegistry> {
    adapterOptions: AdapterOption;
}
interface BelongsToRelationshipMeta {
    type: keyof ModelRegistry;
    options: {
        isRealtime?: boolean;
    };
}
interface HasManyRelationshipMeta {
    key: string;
    type: string;
    options: {
        isRealtime?: boolean;
        buildReference?(db: Firestore, record: unknown): CollectionReference;
        filter?(db: CollectionReference | Query, record: unknown): Query;
    };
}
export default class CloudFirestoreAdapter extends Adapter {
    protected firestoreDataManager: FirestoreDataManager;
    protected referenceKeyName: string;
    protected get isFastBoot(): boolean;
    generateIdForRecord(_store: Store, type: unknown): string;
    createRecord(store: Store, type: ModelSchema, snapshot: Snapshot): RSVP.Promise<unknown>;
    updateRecord(_store: Store, type: ModelSchema, snapshot: Snapshot): RSVP.Promise<unknown>;
    deleteRecord(_store: Store, type: ModelSchema, snapshot: Snapshot): RSVP.Promise<unknown>;
    findRecord(_store: Store, type: ModelSchema, id: string, snapshot: Snapshot): RSVP.Promise<unknown>;
    findAll(_store: Store, type: ModelSchema, _sinceToken: string, snapshotRecordArray?: SnapshotRecordArray): RSVP.Promise<unknown>;
    query(_store: Store, type: ModelSchema, queryOption: AdapterOption, recordArray: DS.AdapterPopulatedRecordArray<unknown>): RSVP.Promise<unknown>;
    findBelongsTo(_store: Store, _snapshot: Snapshot, url: string, relationship: BelongsToRelationshipMeta): RSVP.Promise<unknown>;
    findHasMany(store: Store, snapshot: Snapshot, url: string, relationship: HasManyRelationshipMeta): RSVP.Promise<unknown>;
    protected buildCollectionRef(modelName: keyof ModelRegistry, adapterOptions?: AdapterOption): CollectionReference;
    private addDocRefToWriteBatch;
    private addIncludeToWriteBatch;
    private buildWriteBatch;
    private buildHasManyCollectionRef;
}
declare module 'ember-data/types/registries/adapter' {
    export default interface AdapterRegistry {
        'cloud-firestore-modular': CloudFirestoreAdapter;
    }
}
export {};
